package com.ether;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.Before;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.AbstractApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

public abstract class TestUtils<BEAN>
{
    protected TestUtils(final String beanContextName, final String beanName, final String[] otherContextNames) {
	_beanContextName = beanContextName;
	_beanName = beanName;
	_otherContextNames = otherContextNames;
    }

    protected abstract void beforeBeanTest() throws Exception;

    protected abstract void afterBeanTest() throws Exception;

    @Before
    public final void beforeTest() throws Exception
    {
	Assert.assertNotNull(getApplicationContext());
	Assert.assertNotNull(getBean());
	beforeBeanTest();
    }

    @After
    public final void afterTest() throws Exception
    {
	afterBeanTest();
	_bean = null;
    }
    
    @AfterClass
    public static void afterAllTest() throws Exception
    {
	try
	{
	    if (_applicationContext != null) _applicationContext.close();
	}
	finally
	{
	    _applicationContext = null;
	}
    }

    /**
     * Builds context names from 2 context name arrays.
     * 
     * @param baseContextNames : the base context names.
     * @param otherContextNames : the other context names.
     * @return a context name array.
     */
    protected static String[] buildContextNames(
	    final String[] baseContextNames, final String... otherContextNames)
    {
	int contextNameSize = baseContextNames == null ? 0 : baseContextNames.length;
	contextNameSize += otherContextNames == null ? 0 : otherContextNames.length;

	final String[] contextNames = new String[contextNameSize];

	if (baseContextNames != null)
	    System.arraycopy(baseContextNames, 0, contextNames, 0, baseContextNames.length);

	if (otherContextNames != null)
	{
	    if (baseContextNames != null)
		System.arraycopy(otherContextNames, 0, contextNames, baseContextNames.length, otherContextNames.length);
	    else
		System.arraycopy(otherContextNames, 0, contextNames, 0, otherContextNames.length);
	}

	return contextNames;
    }

    protected final ApplicationContext getApplicationContext()
    {
	if (_applicationContext == null)
	{
	    final String[] contextNames;

	    if (_otherContextNames != null)
	    {
		if (_beanContextName != null)
		    contextNames = buildContextNames(_otherContextNames, _beanContextName);
		else
		    contextNames = buildContextNames(_otherContextNames);
	    }
	    else
	    {
		contextNames = new String[] { _beanContextName };
	    }

	    _applicationContext = new ClassPathXmlApplicationContext(contextNames);
	}

	return _applicationContext;
    }

    @SuppressWarnings({ "unchecked" })
    protected final BEAN getBean()
    {
	if (_bean == null)
	    _bean = (BEAN) getApplicationContext().getBean(_beanName);

	return _bean;
    }

    private static AbstractApplicationContext _applicationContext;

    private String _beanContextName;
    private String _beanName;
    private String[] _otherContextNames;
    private BEAN _bean;
}
